-- Generated by Makefile from the individual Lua test files.


-- test/00_init.lua

-- This will load the new copy of the library on Unix systems where it's built with libtool.
package.cpath = ".libs/liblua-?.so;" .. package.cpath

package.path = "lunit/?.lua;" .. package.path
local lunit = require "lunit"
local TEST_CASE = lunit.TEST_CASE
local is = lunit.assert_equal

local Filter = require "datafilter"

function read_file (filename)
    local fh = io.open(filename, "rb")
    local data = fh:read("*all")
    fh:close()
    return data
end

local function int (x) return x - x % 1 end

local HEX = { "0","1","2","3","4","5","6","7","8","9",
              "a","b","c","d","e","f" }
function bytes_to_hex (s)
    local hex = ""
    for i = 1, s:len() do
        local byte = string.byte(s, i)
        hex = hex .. HEX[int(byte / 16) + 1] .. HEX[int(byte % 16) + 1]
    end
    return hex
end


-- test/10_pristine.lua

local _ENV = TEST_CASE "test.pristine"

function test_no_global_clobbering ()
    local globals = {}
    for key in pairs(_G) do globals[key] = true end

    local lib = require "datafilter"

    for key in pairs(_G) do
        lunit.assert_not_nil(globals[key],
                             "global '" .. key .. "' created by lib")
    end
    for key in pairs(globals) do
        lunit.assert_not_nil(_G[key],
                             "global '" .. key .. "' destroyed by lib")
    end
end


-- test/20_addfile.lua

local _ENV = TEST_CASE "test.addfile"

function test_from_file ()
    local obj = Filter:new("md5")
    obj:addfile("test/data/random1.dat")
    is("a3f8e5cf50de466c81117093acace63a", bytes_to_hex(obj:result()),
       "addfile() on random1.dat")

    obj = Filter:new("md5")
    obj:addfile("test/data/random1.dat")
    obj:addfile("test/data/random1.dat")
    is("b9054bf8fd1941b722ec172efdb7015e", bytes_to_hex(obj:result()),
       "addfile() on random1.dat * 2")

    obj = Filter:new("md5")
    obj:addfile("test/data/random1.dat")
    obj:addfile("test/data/random1.dat")
    obj:addfile("test/data/random1.dat")
    is("b4950de76365266129ccad54c814fddc", bytes_to_hex(obj:result()),
       "addfile() on random1.dat * 3")
end

function test_mix_add_and_addfile ()
    local obj = Filter:new("md5")
    obj:add("string before the file")
    obj:addfile("test/data/random1.dat")
    obj:add("another string between two copies of the random chunk of data")
    obj:addfile("test/data/random1.dat")
    obj:add("some stuff at the end, and this nul should be included: \0")
    is("ef988d269b28ab947437648d63d68a56", bytes_to_hex(obj:result()),
       "addfile() on random1.dat * 2 with strings mixed in")
end

function test_add_from_lua_filehandle ()
    local obj = Filter:new("md5")
    local fh = io.open("test/data/random1.dat", "rb")
    obj:addfile(fh)
    fh:close()
    is("a3f8e5cf50de466c81117093acace63a", bytes_to_hex(obj:result()))

    obj = Filter:new("md5")
    fh = io.open("test/data/random1.dat", "rb")
    obj:addfile(fh)
    obj:addfile(fh)     -- no effect, data already used up
    is("a3f8e5cf50de466c81117093acace63a", bytes_to_hex(obj:result()))
    fh:close()
end

function test_add_from_bad_lua_filehandle ()
    local obj = Filter:new("md5")
    local fh = io.open("test/data/random1.dat", "rb")
    fh:close()
    assert_error("filehandle close", function () obj:addfile(fh) end)
end

local function fake_lua_filehandle_read (self, num_bytes)
    if self.data:len() == 0 then return end     -- EOF
    if num_bytes > self.data:len() then num_bytes = self.data:len() end
    local result = self.data:sub(1, num_bytes)
    self.data = self.data:sub(num_bytes + 1)
    return result
end

function test_add_from_fake_lua_filehandle_table ()
    local data = ("foobar\n"):rep(5000)
    local fh = {
        data = ("foobar\n"):rep(5000),
        read = fake_lua_filehandle_read,
    }

    local obj = Filter:new("md5")
    obj:addfile(fh)
    is("313cf5be140c1ed898c8919454809adc", bytes_to_hex(obj:result()))
end

function test_add_from_fake_lua_filehandle_metatable ()
    local fhmt = {
        data = ("foobar\n"):rep(5000),
        read = fake_lua_filehandle_read,
    }
    fhmt.__index = fhmt
    local fh = {}
    setmetatable(fh, fhmt)

    local obj = Filter:new("md5")
    obj:addfile(fh)
    is("313cf5be140c1ed898c8919454809adc", bytes_to_hex(obj:result()))
end

function test_error_in_fake_lua_filehandle ()
    local fh = { read = function () error"grumpy file handle" end }
    local obj = Filter:new("md5")
    assert_error("file handle 'read' method dies",
                 function () obj:addfile(fh) end)
end

function test_failure_in_fake_lua_filehandle ()
    local fh = { read = function () return nil, "buggerup" end }
    local obj = Filter:new("md5")
    assert_error("file handle 'read' method indicates failure",
                 function () obj:addfile(fh) end)
end

function test_immediate_eof_in_fake_lua_filehandle ()
    local fh = { read = function () return nil end }
    local obj = Filter:new("md5")
    obj:addfile(fh)
    is("d41d8cd98f00b204e9800998ecf8427e", bytes_to_hex(obj:result()))
end

function test_number_from_fake_lua_filehandle ()
    local value = 23
    local fh = { read = function () local r = value; value = nil; return r end }
    local obj = Filter:new("md5")
    obj:addfile(fh)
    is("37693cfc748049e45d87b8c7d8b9aacd", bytes_to_hex(obj:result()))
end

function test_bad_result_from_fake_lua_filehandle ()
    local fh = { read = function () return true end }
    local obj = Filter:new("md5")
    assert_error("file handle 'read' method returns something strange",
                 function () obj:addfile(fh) end)
end

function test_bad_usage ()
    local obj = Filter:new("md5")
    obj:addfile("COPYRIGHT")
    assert_error("addfile() with no args", function () obj:addfile() end)
    assert_error("addfile() with too many args",
                 function () obj:addfile("COPYRIGHT", "extra") end)
    assert_error("addfile() with bad arg", function () obj:addfile(true) end)
    assert_error("invalid input file name",
                 function () obj:addfile("COPYRIGHT\0foo") end)
    assert_error("error opening/reading input file",
                 function () obj:addfile("test") end)

    obj = Filter:new("md5")
    obj:add("foo")
    is("acbd18db4cc2f85cedef654fccc4a4d8", bytes_to_hex(obj:result()))
    assert_error("adding more after finished", function () obj:add("foo") end)
    assert_error("adding more from file after finished",
                 function () obj:addfile("COPYRIGHT") end)
    is("acbd18db4cc2f85cedef654fccc4a4d8", bytes_to_hex(obj:result()))
end


-- test/22_output.lua

local _ENV = TEST_CASE "test.output"

local big_expected = ("YWJjZGVmZ2hpamts"):rep(8192)

function test_output_string_repeatedly ()
    local obj = Filter:new("base64_encode")
    obj:add("foobar")
    obj:add("and some more")
    for _ = 1, 10 do
        is("Zm9vYmFyYW5kIHNvbWUgbW9yZQ==", obj:result())
    end
end

function test_output_simple_big ()
    local input = ("abcdefghijkl"):rep(8192)
    local got = Filter.base64_encode(input)
    is(131072, got:len())
    is(big_expected, got)
end

function test_output_string_big ()
    local obj = Filter:new("base64_encode")
    -- This should force it to reallocate the buffer.  The default buffer is
    -- likely to be in the region of 8Kb, the result of this will be 128Kb.
    for _ = 1, 8192 do obj:add("abcdefghijkl") end
    is(131072, obj:result():len())
    is(big_expected, obj:result())
end

function test_output_filename ()
    local tmpname = os.tmpname()
    local obj = Filter:new("base64_encode", tmpname)
    obj:add("foobar")
    obj:add("and some more")
    obj:finish()
    is("Zm9vYmFyYW5kIHNvbWUgbW9yZQ==", read_file(tmpname))
    assert_error("output sent elsewhere", function () obj:result() end)
    assert(os.remove(tmpname))
end

function test_output_filename_big ()
    local tmpname = os.tmpname()
    local obj = Filter:new("base64_encode", tmpname)
    for _ = 1, 8192 do obj:add("abcdefghijkl") end
    obj:finish()
    local got = read_file(tmpname)
    is(131072, got:len())
    is(big_expected, got)
    assert(os.remove(tmpname))
end

function test_output_filename_write_error ()
    assert_error("trying to write to a directory",
                 function () Filter:new("base64_encode", "test") end)
end

function test_output_file_handle ()
    local tmpname = os.tmpname()
    local fh = assert(io.open(tmpname, "wb"))
    local obj = Filter:new("base64_encode", fh)
    obj:add("foobar")
    obj:add("and some more")
    obj:finish()
    fh:close()
    is("Zm9vYmFyYW5kIHNvbWUgbW9yZQ==", read_file(tmpname))
    assert_error("output sent elsewhere", function () obj:result() end)
    assert(os.remove(tmpname))
end

function test_output_file_handle_big ()
    local tmpname = os.tmpname()
    local fh = assert(io.open(tmpname, "wb"))
    local obj = Filter:new("base64_encode", fh)
    for _ = 1, 8192 do obj:add("abcdefghijkl") end
    obj:finish()
    fh:close()
    local got = read_file(tmpname)
    is(131072, got:len())
    is(big_expected, got)
    assert(os.remove(tmpname))
end

local function fake_file_handle_write (self, arg, extra)
    assert_string(arg)
    assert_nil(extra)
    self.data = self.data .. arg
    return true
end

function test_output_fake_file_handle ()
    local fh = { data = "", write = fake_file_handle_write }
    local obj = Filter:new("md5", fh)
    obj:add("foobar")
    obj:finish()
    is("3858f62230ac3c915f300c664312c63f", bytes_to_hex(fh.data))
end

function test_output_fake_file_handle_mt ()
    local fhmt = { write = fake_file_handle_write }
    fhmt.__index = fhmt
    local fh = { data = "" }
    setmetatable(fh, fhmt)
    local obj = Filter:new("md5", fh)
    obj:add("foobar")
    obj:finish()
    is("3858f62230ac3c915f300c664312c63f", bytes_to_hex(fh.data))
end

function test_output_fake_file_handle_fail ()
    local fh = { write = function () return nil, "buggerup" end }
    local obj = Filter:new("md5", fh)
    obj:add("foobar")
    assert_error("'write' method reports failure",
                 function () obj:finish() end)
end

function test_output_fake_file_handle_error ()
    local fh = { write = function () error"grumpy write method" end }
    local obj = Filter:new("md5", fh)
    obj:add("foobar")
    assert_error("'write' method dies", function () obj:finish() end)
end

function test_output_fake_file_handle_bad_write_method ()
    local fh = { data = "", write = 123 }
    assert_error("'write' method is not a function",
                 function () Filter:new("md5", fh) end)
end

function test_output_function ()
    local got = ""
    local func = function (data) got = got .. data end
    local obj = Filter:new("base64_encode", func)
    obj:add("foobar")
    obj:add("and some more")
    obj:finish()
    is("Zm9vYmFyYW5kIHNvbWUgbW9yZQ==", got)
    assert_error("output sent elsewhere", function () obj:result() end)
end

function test_output_function_big ()
    local got = ""
    local func = function (data) got = got .. data end
    local obj = Filter:new("base64_encode", func)
    for _ = 1, 8192 do obj:add("abcdefghijkl") end
    obj:finish()
    is(131072, got:len())
    is(big_expected, got)
end

function test_output_function_error ()
    local func = function (data) error"grumpy callback function" end
    local obj = Filter:new("base64_encode", func)
    obj:add("foo")
    assert_error("callback throws exception", function () obj:finish() end)
end

function test_bad_usage ()
    assert_error("too many args",
                 function () Filter:new("md5", nil, "foo") end)
    assert_error("invalid algo name", function () Filter:new("md5\0foo") end)
    assert_error("unknown algo name", function () Filter:new("erinaceous") end)
    assert_error("invalid output file name",
                 function () Filter:new("md5", "tmpname\0foo") end)
    assert_error("error opening output file",
                 function () Filter:new("md5", "test") end)

    local obj = Filter:new("md5", function () end)
    obj:add("foo")
    obj:finish()
    assert_error("calling finish twice", function () obj:finish() end)
end

function test_reusing_file_handle ()
    local tmpname = os.tmpname()
    local fh = assert(io.open(tmpname, "wb"))
    local obj = Filter:new("base64_encode", fh)
    obj:add("foobar")
    obj:finish()
    fh:write("\n")
    obj = Filter:new("base64_encode", fh)
    obj:add("frobnitz")
    obj:finish()
    fh:write("\n")
    fh:close()
    is("Zm9vYmFy\nZnJvYm5pdHo=\n", read_file(tmpname))
    assert(os.remove(tmpname))
end


-- test/24_options.lua

local _ENV = TEST_CASE "test.options"

function test_module_metadata ()
    is("string", type(Filter._VERSION))
    assert(Filter._VERSION:len() > 0)
    is("datafilter", Filter._NAME)
end

function test_ignored_options_simple ()
    local input = "foo"
    local expected = "acbd18db4cc2f85cedef654fccc4a4d8"
    is(expected, bytes_to_hex(Filter.md5(input)))
    is(expected, bytes_to_hex(Filter.md5(input, nil)))
    is(expected, bytes_to_hex(Filter.md5(input, {})))
    is(expected, bytes_to_hex(Filter.md5(input, { foo = 23, bar = "baz" })))
end

local function do_md5_simple (...)
    return bytes_to_hex(Filter.md5(...))
end

function test_ignored_options_oo ()
    local input = "foo"
    local expected = "acbd18db4cc2f85cedef654fccc4a4d8"
    is(expected, do_md5_simple(input))
    is(expected, do_md5_simple(input, nil))
    is(expected, do_md5_simple(input, {}))
    is(expected, do_md5_simple(input, { foo = 23, bar = "baz" }))
end

function test_bad_useage_simple ()
    local input = "foo"
    assert_error("not enough args", function () Filter.md5() end)
    assert_error("too many args", function () Filter.md5(input, nil, true) end)
    assert_error("bad options value (true)",
                 function () Filter.md5(input, true) end)
    assert_error("bad options value (false)",
                 function () Filter.md5(input, false) end)
    assert_error("bad options value (string)",
                 function () Filter.md5(input, "foo") end)
    assert_error("bad options value (num)",
                 function () Filter.md5(input, 23) end)
end

function test_bad_useage_oo ()
    assert_error("not enough args", function () Filter:new() end)
    assert_error("too many args",
                 function () Filter:new("md5", nil, nil, true) end)
    assert_error("bad options value (true)",
                 function () Filter:new("md5", nil, true) end)
    assert_error("bad options value (false)",
                 function () Filter:new("md5", nil, false) end)
    assert_error("bad options value (string)",
                 function () Filter:new("md5", nil, "foo") end)
    assert_error("bad options value (num)",
                 function () Filter:new("md5", nil, 23) end)
end

function test_exception_in_algo_oo ()
    -- I need to make sure that this exception doesn't cause a leak.
    local obj = Filter:new("base64_decode", nil,
                           { allow_whitespace = false })
    assert_error("whitespace not allowed, options passed to :new()",
                 function () obj:add("eH l6"); obj:result() end)
end

function test_handle_init_error_oo ()
    -- The non-OO version of this is tested in each algorithms tests.
    local options = { line_ending = true }
    assert_error("bad type for line_ending option",
                 function () Filter:new("base64_encode", nil, options) end)
end


-- test/40_adler32.lua

local _ENV = TEST_CASE "test.adler32"

local misc_mapping = {
    -- Test data from the test suite of the Digest::Adler32 Perl module
    [""] = "00000001",
    ["a"] = "00620062",
    ["abc"] = "024d0127",
    ["abcabc"] = "080c024d",
    [("\255"):rep(32)] = "0e2e1fe1",
}

-- Data calculated with a bit of Perl code (in the file 't/adler32-gen.pl')
-- where each entry in the array is for a chunk of data of as many bytes as its
-- index, and the bytes all progress in a simple way.
local progressive_adler32_expected = {
    "00080008",
    "002e0026",
    "0089005b",
    "013000a7",
    "023a010a",
    "03be0184",
    "05d30215",
    "089002bd",
    "0c0c037c",
    "105e0452",
    "159d053f",
    "1ae00543",
    "203e055e",
    "25ce0590",
    "2ba705d9",
    "31e00639",
    "389006b0",
    "3fce073e",
    "47b107e3",
    "5050089f",
    "59c20972",
    "641e0a5c",
    "6e7b0a5d",
    "78f00a75",
    "83940aa4",
    "8e7e0aea",
    "99c50b47",
    "a5800bbb",
    "b1c60c46",
    "beae0ce8",
    "cc4f0da1",
    "dac00e71",
    "ea180f58",
    "fa6e1056",
    "0ae8106b",
    "1b7f1097",
    "2c5910da",
    "3d8d1134",
    "4f3211a5",
    "615f122d",
    "742b12cc",
    "87ad1382",
    "9bfc144f",
    "b12f1533",
    "c75d162e",
    "dd9d1640",
    "f4061669",
    "0abe16a9",
    "21be1700",
    "392c176e",
    "511f17f3",
    "69ae188f",
    "82f01942",
    "9cfc1a0c",
    "b7e91aed",
    "d3ce1be5",
    "efc21bf4",
    "0beb1c1a",
    "28421c57",
    "44ed1cab",
    "62031d16",
    "7f9b1d98",
    "9dcc1e31",
    "bcad1ee1",
    "dc551fa8",
    "fcdb2086",
    "1e65217b",
    "3fec2187",
    "619621aa",
    "837a21e4",
    "a5af2235",
    "c84c229d",
    "eb68231c",
    "0f2923b2",
    "3388245f",
    "58ab2523",
    "7ea925fe",
    "a59926f0",
    "cc9226f9",
    "f3ab2719",
    "1b0a2750",
    "42a8279e",
    "6aab2803",
    "932a287f",
    "bc3c2912",
    "e5f829bc",
    "10842a7d",
    "3bd92b55",
    "681d2c44",
    "94672c4a",
    "c0ce2c67",
    "ed692c9b",
    "1a5e2ce6",
    "47a62d48",
    "75672dc1",
    "a3b82e51",
    "d2b02ef8",
    "02752fb6",
    "3300308b",
    "64773177",
    "95f1317a",
    "c7853194",
    "f94a31c5",
    "2b66320d",
    "5dd2326c",
    "90b432e2",
    "c423336f",
    "f8363413",
    "2d1334ce",
    "62b335a0",
    "993c3689",
    "cfc53689",
    "067436a0",
    "3d4236ce",
    "74553713",
    "abc4376f",
    "e3a637e2",
    "1c21386c",
    "552e390d",
    "8ef339c5",
    "c9873a94",
    "05103b7a",
    "41873c77",
    "7e123c8b",
    "bac83cb6",
    "f7c03cf8",
    "35203d51",
    "72e13dc1",
    "b1293e48",
    "f00f3ee6",
    "2fb93f9b",
    "70204067",
    "b16a414a",
    "f3ae4244",
    "36124255",
    "788f427d",
    "bb4b42bc",
    "fe5d4312",
    "41eb437f",
    "85ee4403",
    "ca8c449e",
    "0feb4550",
    "56044619",
    "9cfd46f9",
    "e4ed47f0",
    "2cfa47fe",
    "751d4823",
    "bd7c485f",
    "063d48b2",
    "4f59491c",
    "98f6499d",
    "e32b4a35",
    "2e1e4ae4",
    "79c84baa",
    "c64f4c87",
    "13d94d7b",
    "615f4d86",
    "af074da8",
    "fce84de1",
    "4b284e31",
    "99c04e98",
    "e8d64f16",
    "38904fab",
    "88e75057",
    "da01511a",
    "2c0451f4",
    "7ee952e5",
    "d1d652ed",
    "24f1530c",
    "78335342",
    "cbc2538f",
    "1fc453f3",
    "7432546e",
    "c9325500",
    "1eea55a9",
    "75535669",
    "cc935740",
    "24d0582e",
    "7d035833",
    "d552584f",
    "2de35882",
    "86af58cc",
    "dfdc592d",
    "399059a5",
    "93c45a34",
    "ee9e5ada",
    "4a445b97",
    "a6af5c6b",
    "04145d56",
    "616c5d58",
    "bedd5d71",
    "1c8d5da1",
    "7a755de8",
    "d8bb5e46",
    "37855ebb",
    "96cc5f47",
    "f6b65fea",
    "576960a4",
    "b8de6175",
    "1b4a625d",
    "7ea6635c",
    "e2186372",
    "45c6639f",
    "a9a963e3",
    "0df6643e",
    "72a664b0",
    "d7df6539",
    "3dc765d9",
    "a4576690",
    "0bc4675e",
    "74076843",
    "dd46693f",
    "46a76952",
    "b023697c",
    "19ef69bd",
    "84046a15",
    "ee886a84",
    "59a16b0a",
    "c5486ba7",
    "31b26c5b",
    "9ed86d26",
    "0cef6e08",
    "7bf06f01",
    "eb016f11",
    "5a486f38",
    "c9be6f76",
    "39986fcb",
    "a9cf7037",
    "1a9870ba",
    "8bec7154",
    "fdf17205",
    "70cd72cd",
    "e47973ac",
    "592a74a2",
    "cdd974af",
    "42bb74d3",
    "b7c9750e",
    "2d387560",
    "a30175c9",
    "19597649",
    "903976e0",
    "07d6778e",
    "80297853",
    "f958792f",
    "73897a22",
    "edb57a2c",
    "68117a4d",
    "e2967a85",
    "5d797ad4",
    "d8b37b3a",
    "54797bb7",
    "d0c47c4b",
    "4dc97cf6",
    "cb817db8",
    "4a217e91",
    "c9a27f81",
}

function test_trivial_obj ()
    local obj = Filter:new("adler32")
    is("00000001", bytes_to_hex(obj:result()))
    local obj = Filter:new("adler32")
    obj:add("")
    is("00000001", bytes_to_hex(obj:result()))
end

function test_misc_adler32 ()
    for input, expected in pairs(misc_mapping) do
        local got = Filter.adler32(input)
        is(4, got:len())
        is(expected, bytes_to_hex(got),
           "Adler32 of " .. string.format("%q", input))
    end
end

function test_gradual_size_increase ()
    local input = ""
    local byte = 7

    for i = 1, #progressive_adler32_expected do
        local expected = progressive_adler32_expected[i]

        input = input .. string.char(byte)
        byte = (byte + 23) % 256
        assert(input:len() == i)

        local got = Filter.adler32(input)
        is(4, got:len())
        is(expected, bytes_to_hex(got),
           "Adler32 of " .. string.format("%q", input))
    end
end


-- test/40_md5.lua

local _ENV = TEST_CASE "test.md5"

local misc_mapping = {
    -- Test data from RFC 1321, appendix 5
    [""] = "d41d8cd98f00b204e9800998ecf8427e",
    ["a"] = "0cc175b9c0f1b6a831c399e269772661",
    ["abc"] = "900150983cd24fb0d6963f7d28e17f72",
    ["message digest"] = "f96b697d7cb7938d525a2f31aaf161d0",
    ["abcdefghijklmnopqrstuvwxyz"] = "c3fcd3d76192e4007dfb496cca67e13b",
    ["ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789"]
        = "d174ab98d277d9f5a5611c2c9f419d9f",
    ["123456789012345678901234567890123456789012345678901234567890123456" ..
     "78901234567890"] = "57edf4a22be3c955ac49da2e2107b67a",
}

-- Data calculated with a bit of Perl code (in the file 'test/data/md5-gen.pl') where
-- each entry in the array is for a chunk of data of as many bytes as its
-- index, and the bytes all progress in a simple way.
local progressive_md5_expected = {
    "89e74e640b8c46257a29de0616794d5d",
    "7240e7b150d6025912db8cdcb3feebeb",
    "88301910845f8f4b88aeedce3cb1c4c9",
    "31c359a107aa85d5482e59ed5a24a1c2",
    "8fc12582721c9049c94b319633573f0f",
    "5b8051a13d6d44e031cab9cd2e9eb228",
    "2779219adac2242744ca765699b84df9",
    "c3959ba26023891d050d42792e5e173a",
    "15f44f8d145d043172477e36f3a1f346",
    "8a3e55ec95e1a3ef23c79c52d1bf72cc",
    "d7d031d1387e5313c1a7eb02fc836166",
    "98897f26186be77797e29994804e5b93",
    "268651a5a2ae63a78db619877025d4fd",
    "8d391ebc78bc811e4c5c022aabc625b5",
    "e5a5ce523ec61bedc0d212b053116fd3",
    "2e9ccae274ee36377d9227cf6b71976b",
    "8a3b44b957e8f1af8a4129172fa91382",
    "a8c38b5fce9ce49f4f261926aa00426a",
    "4b97d7d215627ece5ddc18f6b3813235",
    "fcd6a7fdbb76fd83e5f4d43ce6380666",
    "6b7ec7c363e76265b6a8c930747bafa2",
    "395662ec023352ddab366bbdcf9d0042",
    "cc8adca2bf689eb3707952766c426f21",
    "4db9d99d0f0d71de20d8aff2b04f4ac4",
    "3dafdeec56a9db606917a825d66ada8f",
    "efb2d65eb1a3706dbd6bba6f40b05dce",
    "71ee81fc27e7f5f5882c74b619f7150f",
    "cf493506aaca99256856170f7b0257ad",
    "b4b9e99701ef0fc313cde858c891b311",
    "af76524affbc14020a180998faa89de2",
    "218703b715d947b23ff185b402063d97",
    "dcc961633d2738de9e898227e276e789",
    "7c8a68edf6fb40763d6c0ff951b21960",
    "e0afa0d7984bcfaf953edc472eaf6605",
    "aa0e5cdfe2ff22214fff041f6233cdbe",
    "bd9485ea5a5610b7d314579f106a5161",
    "c7e79c6ff30732a0ac3bd257d5ced98d",
    "08a98a8a7ea8ea6c7b310249c03b7121",
    "a44f7ae0f2ef8d815451440eb6f594f6",
    "17b7f75403eee0c0eff3f2f0928a7f60",
    "1da61992f1123a0872d00a50452aa174",
    "9406acf29fbcc711501b3e228699c5ab",
    "703eaf26a15d172992e186ddf76a842c",
    "b2576c8ce4ca12a292c7b5fdd92ffba4",
    "6bcc790decd1e629c3141c63d9f98554",
    "8febf3e7555aa2fb79b691b83a05d57b",
    "95048660de69717792bbdb4da92412db",
    "f9b9f00f82d5a2a2fa3b860da82a416a",
    "b6b522e3bfc6ce9f0873c212506f0b21",
    "c2d131765ab338d4e3faa8f768325a86",
    "7a9760954647ad443dbfe8e8a106b821",
    "030ea05631f3b82bf1872849a80ac5ae",
    "add8b85f476f52b35401c5d34f145bb9",
    "13b53ac4c6e58605694e26e79c072935",
    "d27d8878b56868d8a9090170d2acfbce",
    "cda66572dc870cf8c6cacd07811ad272",
    "6ae022259bc385938b3e65a2954832c9",
    "9c13a3a772f1260c13d1326bfff573fa",
    "99ecfa70f7bfe0a2daf7b77838233b12",
    "5f3ca466ea1f09590f8dbf87aaf08e5d",
    "13f7c93d0d3f6854dfffc62b621b3c99",
    "e92d35916e0de3a03c6b577885eda22d",
    "fa1ad25ec5960f37cfbe249d441b3241",
    "f5fef1c74c86e2b6fd68199458bacd59",
    "f4bdcb5b126b8b6497a718301334c18b",
    "0b10e316a099c5211eac659652ce2f2d",
    "7e521d71c4774bd587b461daa8076958",
    "b9048f36095d24a8ced038b78023fa52",
    "7bf3e3ffb5cb08af884c3a59758acff1",
    "4600da41d1d93c746a81c6dfc9eaf219",
    "534a6b468a183200b0e547299c99debb",
    "d9e1b9269425e714fb1fa088510e871c",
    "0afac272e4150ab42f7bc9f116ccc3a7",
    "9db432077aedcb612869e1797adc5764",
    "016b45803f2532d6051d8636629a9f0e",
    "222b1092bbae19a524c98b27a1cac1b6",
    "8811ce6a671596f2bf80aa89239d2525",
    "c364f70cec48d333c179ee3fe754ec2e",
    "d3172a6e1a662684b1cbb2978d60340f",
    "3120d9594b6c3755d6feb071f51e46fc",
    "b4a2a933067d40a89a5186967e3480f2",
    "2af7fe12ff43a083e800046220fa6c9e",
    "9779268862bb6738755a0ba31b2dd84c",
    "a756bb43d83e3f74edf4ad5945c2485d",
    "33164f9bd9f826e8c9446d32bad22acf",
    "abb34d9a08c3d7ab2a7e0db8c8559758",
    "d3dd6c4507f9ed19677564df2733d4a3",
    "757f8d9400fb9bfb51fe9a4bc62e1c22",
    "3bc0daf3468eb31e43b352e10dc824d6",
    "9ed80da75621199fde01f74707d91f26",
    "73dbef39378325090f3d006c9c240d09",
    "17cc6ac81aa6d5080cfafe69c652390b",
    "2565dd7386ca098be34c49dbaa01457d",
    "18c560001385f674d76cb8da239fcf89",
    "35ec4289ea9c1a7daa6f31f38bb7741b",
    "34d3ce9550688f418d885dd1ea5c83c0",
    "0664070fa900c0f3ac0b8d449d0337a8",
    "81b2c5767ee92db010c8fded0cc85b74",
    "39527094c6ba2f482186428a22ba8f7f",
    "0c2256bc176b7aa72c64703cc62f4756",
    "dc863c6ca6ad878bcd54ee1ff127f8c2",
    "d69c27e91fca527ca2659a6ccf697673",
    "fb90adbfaf09acb14076cb69d80d0e9b",
    "94d3e69b241db8e06b9383e54eda394b",
    "aa944a5000e19b11d00392f6d8bfa886",
    "ad35d72d040a5f619edc921532009c87",
    "0fbc6e5fa24826dcaca7e7e30f3f67a2",
    "a1b4ad040e07ad59b6a97da742aabf26",
    "6a21e824400f8a528be092cb3448c2ae",
    "625e8e0f736815e290290a8639f68327",
    "5978c9abbc134c33ecabf361296699a5",
    "f52df035041d7b35eac734708dd92519",
    "22ffbe6b5d7119a2c72b23fbf6f11644",
    "14239de98026c29da03065f4780e6235",
    "daf655d8b63698885faba199fd2eef15",
    "3561bd60a6dadcc6811fa4fe448a597f",
    "150cab16147561855d4eaf2208a45101",
    "4d72ee11cd0b6c1dd9863eeeaceeaf6d",
    "df32d4bf0ffe6780cf09413ac70bea48",
    "55c86558b71f037d3af8476bbd011e94",
    "8b39ab3533a75cc6e045d4cb4e803892",
    "36eceeeb06c4fa211363cd246c908628",
    "520f7de7fe76c02db9723d07eb8e2f6d",
    "5c448a4977f677d7af528086c83f7a06",
    "14a4b79836438b7b8913dd6499699c5f",
    "f5ff9129e9d58ec31b01694edd3f3421",
    "677f855124ab0f09484592d06bd2d96b",
    "4effcd63d3c8db61da2a9a23e01c48db",
    "6bd9aba84ebbc47306381540dc515521",
    "d886759dc59f4a8c2239fb8828031843",
    "a96c73c431c801a66524a3aad930cc58",
    "4e6af120c39700c6f2e141339ddaee4a",
    "b4212f4b923c979c249d88e95cc6baa1",
    "a123e4d9fcf32cd2a1b1c33c4eb54146",
    "755562bca2168a1462c19d5eed0cc493",
    "2984b42c59e9fdbcdddebda914e7f731",
    "5a7452ba8cc152ef551b76c5746dee60",
    "069ebbb114964569ea16931ed39eac07",
    "66995604ff177a0350408b3c720d76f9",
    "8e9fb5f02794c0686fdab8bf0048277f",
    "f98b787db9f5035ef4bbad4c1de84c5e",
    "05e76be9a341446b8da455c324eaca40",
    "0663cf543e1095b6831746b72ee62183",
    "5b24404e7db0788d41145eedac42f12e",
    "c9fadb88620420eaad1964e30302ecf1",
    "2160db611647b47b12fbce9b1c15936b",
    "1a7256cc76c0ca7c350c7fbe39690a44",
    "7777560d615e364d69219ce9a3c698c1",
    "c1f1f7bef53d2b1b3800ca0dba9fd410",
    "5c50f06f4dbbed981ec6b647cdd1b450",
    "906d23fb646141fe73c1838ba169abe4",
    "878d660f18b1e74b386593267e249a1c",
    "cee962c3ccb39973d2264dcbd30926b5",
    "5b7b3bbea3fce59ae7a750127b813e04",
    "e23336f40f486279d474a6d77a49341c",
    "4a4cce96811aa9afd1e180090030533b",
    "84553e93d5be54f1ea047f6bb961f8d4",
    "3476288a8bcb378ea6de2372fd7fac30",
    "d45996ba7fd7cc6200a6a40c76d778f3",
    "02dd6a93c83f5534fb2b4062f51cae3a",
    "7910d3a17229cea2bac5e78443b97ade",
    "ed7a2f386e0a4ba07ac6e1ee8c639dbb",
    "667e4f19157901842877c35fce89083c",
    "9a698be051055e6ff73689ccac1a258e",
    "d7ba14d005e849431ac930914758afe4",
    "1cffc4022ca9d302b67556d36b3c2aaa",
    "7d77de2776230c65dd3b39a593cf420a",
    "64cdfa240fd3cf893b4af90d35d12905",
    "b61d531d8f3ef1f6e397ee6a23f2a17f",
    "9c0117dfa32108c73d1558d82acca89d",
    "9aa18cb12af86cd5e537e7893cbbf1c9",
    "0627f83406567a3aef21e31a0fd7d55e",
    "f448d064a92a14bee050b4be96262ad4",
    "a8743a010ad33108d2499476c5c77113",
    "568d41f07d759b59bdb93c93c107dd7d",
    "e01c1f055443470c064ebcf40341bfd7",
    "36df30f18205160d2df9286685ce8265",
    "9da39df6efd0767b4cb2033a88c056e2",
    "1a2e56248e5ee0a409259b7a278f1e8b",
    "1dc38a078f43e10dbd2659aaa3420654",
    "2362350e33dbf86e402023f61b7cc899",
    "b29fe1bded2355333f0dcde7d8b569b4",
    "d970b54334c07178332196b231ef0d7d",
    "6d98d6cd5b8f7a33ee0e1255874c7e58",
    "c6433c876c87f03e3699df398899ea3f",
    "9000ef0c3aded62a32f19715a87b38de",
    "b00203122224fe9f8401dbd482b2de7d",
    "da520090e9d8ddb3b72f7d26fd7f1a9b",
    "3c11d3f66822589d4aeda93fdd7fb90e",
    "5c0d64db5893df7309d4df7385fcc984",
    "447a9c90ede2b94599d43498a20b5193",
    "dc7d36f8b2405f367b6321e7f02aef7b",
    "f511a8e1d0760453ba7ce95363d024b2",
    "c34ed9722a23c675a0ba6270e9deb1bb",
    "89897279a6a5da68c73028090b3b9d7b",
    "5f210a8ffda2781c865753e3b65257e3",
    "18cdff7238819f207c4f31348d4c3e3a",
    "60782df42f86f69a498db229fd6616d8",
    "37de7e5e4b19356ec4021535801edb10",
    "4eb176fcd66b0d779361fadaed089edd",
    "9cb03eceeb411da026432444b82d2ea6",
    "b2438fd57c7e6e735e109199a84eecb8",
    "1890e73ac2bffc95faa081e0303e93b2",
    "dc18e4de730ae242a0fcb3f29ce0394b",
    "262e2d3ee458a81e8b8a42db96160d26",
    "4310f1f8cac06414aeb0a1372506ba7b",
    "d399e358932e7b543009bb7cea5bf124",
    "bfaa951cd2bb24516af8a04ecd00acda",
    "3828cf16d88a51b6652d9261b0b0619b",
    "ae72263136bae239b9b431b1edfd2891",
    "b8a84ab89e7ebdd4f82498fd8411c146",
    "2e15e52184777f43f6a53798b6afebf0",
    "ba5345e9d7a2672f04b0cc16dab13a6b",
    "92f9d9831c98b12225700d5301a82e98",
    "a18268dd4ae2ea21d42598f34a1b919f",
    "277331ef82901d40b865db4970690787",
    "848436e76a90a95cf2b54dfde48d032c",
    "fa3008030775fd3452d315554b3efe05",
    "4b635263a641766a29fd25e8c985e295",
    "dfc4179829f06949093ad747185bc8e6",
    "f4c390915048a237ae5ec8e91bbe4257",
    "46ec9286edf72da52660ddcfe86e6056",
    "8b15a64cbcf3674a5544e7cc21f77006",
    "f9a7199bbb14868f4c25d31a965506cc",
    "547dd9303985b32a34a8cc1fe9c48324",
    "aa4c9767459152227ab8c0b9665776c6",
    "fd031730fe839fa1622a7a2be544c3eb",
    "9a414f7377e9d80008041575c7381a77",
    "25aa5ef1e6f92bdfe3831b621199544d",
    "88b205f5e94305b3414ff8aa5645b5ec",
    "860c4611596ec9f9008023a3bb1f814e",
    "4eb1e7d69368b90eac18fc335ca60e4c",
    "78efc24bbd5426a1106a9124b60a2bcc",
    "9586174868369fd3ceb217988d1c33cc",
    "644b978ce33d2fb14c62767b24c1d99a",
    "4e4ee1f8a3d967e0e859eee885b9e25c",
    "717ad34511fd792ee167c21616c30546",
    "cd47470766f7420b9dd5611e8411f248",
    "e12c2eea66fb3a61e7564009d5a9b859",
    "e445823deb473c71026b98954ad327a3",
    "ba76b70939068630a284d873e2aae20a",
    "0aee9c1d179a7a03e7c35cf5ab9cd7d2",
    "faed0a07af39c467e435195336860bf4",
    "8f5d742e4ca10b1d325ee62312db06f5",
    "cf6c490d38c2b9621886931610bfc95a",
    "cdc4b0657afeaca5ff2c7f7284ca1bf9",
    "e6f5db502a1fb7caab0ba697eef4d6f1",
    "cc8f8e8f3d25540ed862dcb22d7bffc0",
    "5249a8cc3add3daf17d5f045056a28ef",
    "b5c4c6fd09829184c171639f51d70c6e",
    "2ba1290ac9151e403e26c2d43a18fa35",
    "57859e4ce1ad4ad7d52241738f93c2c7",
    "5f7727f587baca10b1d7073568bef265",
    "fda4ee8777d7551bbca7f2224f3681fc",
    "9abd143326bf7260b56acf19b8cef39e",
    "f9b574526cc9a4528d45f8816706fa83",
}

function test_trivial_obj ()
    local obj = Filter:new("md5")
    is("d41d8cd98f00b204e9800998ecf8427e", bytes_to_hex(obj:result()))
    local obj = Filter:new("md5")
    obj:add("")
    is("d41d8cd98f00b204e9800998ecf8427e", bytes_to_hex(obj:result()))
end

function test_misc_md5 ()
    for input, expected in pairs(misc_mapping) do
        local got = Filter.md5(input)
        is(16, got:len())
        is(expected, bytes_to_hex(got),
           "MD5 of " .. string.format("%q", input))
    end
end

function test_gradual_size_increase ()
    local input = ""
    local byte = 7

    for i = 1, #progressive_md5_expected do
        local expected = progressive_md5_expected[i]

        input = input .. string.char(byte)
        byte = (byte + 23) % 256
        assert(input:len() == i)

        local got = Filter.md5(input)
        is(16, got:len())
        is(expected, bytes_to_hex(got),
           "MD5 of " .. string.format("%q", input) .. ", with i=" .. i)
    end
end


-- test/40_sha1.lua

local _ENV = TEST_CASE "test.sha1"

local misc_mapping = {
    [""] = "da39a3ee5e6b4b0d3255bfef95601890afd80709",
    -- Test data from RFC 3174, section 7.3
    ["abc"] = "a9993e364706816aba3e25717850c26c9cd0d89d",
    ["abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq"]
        = "84983e441c3bd26ebaae4aa1f95129e5e54670f1",
    [("a"):rep(1000000)] = "34aa973cd4c4daa4f61eeb2bdbad27316534016f",
    [("0123456701234567012345670123456701234567012345670123456701234567"):rep(10)]
        = "dea356a2cddd90c7a7ecedc5ebb563934f460452",
}

-- Data calculated with a bit of Perl code (in the file 't/sha1-gen.pl') where
-- each entry in the array is for a chunk of data of as many bytes as its
-- index, and the bytes all progress in a simple way.
local progressive_sha1_expected = {
    "5d1be7e9dda1ee8896be5b7e34a85ee16452a7b4",
    "654d40aef7d1d67701ffb0ea3d360ec96b1a9d9a",
    "9efb20f73c11e47be0d0ed56607a9a3ee953d224",
    "abc1b679fc953dad0ecb436c4fb9abc6529562b2",
    "ab3bc3943c25939c5d718893e56759eed6547ea3",
    "4aea5d647c96c6db2a53be0f00d7207ff19e980d",
    "33215c2dc757ff5121052a707199c613709d9020",
    "2afbe546d4331073c16580bc602836c9b4297579",
    "64bab3e64fc8dbc59f92f37270f3f059d8bab66a",
    "e4fefe6eac630c1aa8c681bb14687bcae0b5e773",
    "076093a117d310835e06881d0d2b8eb09d89e1cc",
    "207b47f220361085bf719057314029098a679bcb",
    "e1b554be426f494375b1df2ca7653045bade9361",
    "528cf67f267ad97b6f3f05887823fe52004c859e",
    "04a1da4aacd8df254fb6d54b039d525281a0234d",
    "5e64dddc792568b2cd97af8c4f8a4f53d19d1bb7",
    "a22ac2c0943f96f78667964e36e9d63738f69d65",
    "070a032aae52eced85fb1d02028669980d741252",
    "4370781047c0afd2fb1c070846ced5341f9a764d",
    "261c80bfa98bc9ea65572735fa975f958aa5c434",
    "0304a272aec8006949bf9f822ce8402dd2b6c9af",
    "bf79eb2d9b1ac6ec45c5e7115f91055af0bbc67b",
    "333f7f2d4cff0a67adfdda5022d54b094dabba1a",
    "da9d10053a4fd48446338e283b67282eed5dd4a0",
    "65cc462f12791862b07b62419c09d43cb6b91fe7",
    "f94bb93b4bcd3609dce0c692be83f41290576b0f",
    "27fc05a018e42546e89e05737793e44a2c8202af",
    "1dd41f2854d4876d951c7f96bc9724954ee9c057",
    "7b34c18610fb47d8e36bbee6b17eb5f8f2397b77",
    "443086c22737a7f7b15371709f592712ced33422",
    "a74f2ff8de85fbd3ad002555feda90aee633cf6d",
    "047095c1601086e17119892c45b8e96e55b6ba75",
    "72cf24cd2857f520fd5184e7c904c6300311d83f",
    "9b0427f4439cb9bc835d6e123d45dfb78ae5e4e5",
    "edc6eb7289c9c29a7f61db360490991a390b6fc1",
    "e9d08420ebe9cb07584e5e3b904d76c6f12e02a1",
    "617c3b880fe9f9a55d06af8571ea4b34831ba684",
    "44806b1a22ec1144b027061d41d2d84d330c28f4",
    "636e896caa9f205c68511b35eceb8a1e0e095c93",
    "8eda73804191e03e4922c61d697f9b34757a6fdf",
    "7ddff836bf6417c01014b6e90847bf441b8f455d",
    "c43aec0c274ad3bc7b29f16c54dd8559c4dab19d",
    "606ad1adc976684c9c92a6ab1af0991bb0fc4ca4",
    "69295f6b17e87135661ebd4493c56f94a474d183",
    "1cbde81ffed2b4a80d65633930fe33ef1a627175",
    "8def203915e0ae51aac48cfedae5666de885376d",
    "a7f35812caaa5c1192078a761efd5a2086973380",
    "00878b70188a8b93e6a604944478892583d32dea",
    "21ceaa1ebbf27e6dc590fbc679564138ec1df19c",
    "b461e1ce49bab701ec464dd85e7ab91b38379fb3",
    "4be307d6c2e7aa3837f41de264579c74f2e238e9",
    "0e5de862ad212f35607cc4cf47232f0b230b5130",
    "a5462b7801845410e3a551ea518d01062f72eaa4",
    "0e25542b297194a560029307ccd73dbbfa3dcd7f",
    "189aea94dbc8d1823d4a2f5f7a34fbedebc72334",
    "79c78c1949a7a466ed49a3e0fab20620cb17d1ca",
    "3fcdf223a12255d50b88a02726800177fec35e11",
    "ed6529020471ac0354a3f6b84ce11335ad0f3325",
    "1dd1508955d62d4bdf7881ec20a41ffb5491e2fa",
    "fd86e02c6b9ed457addc436eaa3a3237b15ef547",
    "2ed6b739670793412ba6363ceb4fc627ef9d4cf7",
    "b7ee7e72cbc70f0b8758f863a32e0df0bf35a57b",
    "25133d08da61d108171e641b12b82594d1208d54",
    "55370286b097d147abeff481141803fd07027094",
    "0606f0296fb71f47eacd1f62d45e472423a51898",
    "650e1934b4977bfab3008240a2caaa6b08131e98",
    "ed1de3766297e71519d3cc2ce865e38fa28c31ed",
    "cab8c35e8caa3a8ff6c3333096c2b18649dac3c5",
    "69768d1311f2d4de70b232744f8e46599ebca303",
    "f7d8537833081c249f94190ed0ae0e32d619f0cf",
    "c88f7da4a2aa2bcd607a82d69828353b33da2869",
    "98bfa299d2ad54de5bbf0e21bde43de6893a7aa8",
    "937ef7656bd8ee182c92e425ae96f860edfdd1a5",
    "51b289b7cb999b5a180ed22b241d51523fcc8be3",
    "7ef6afac9559229e7148ee3f1c204bd15aa3e48c",
    "bc845a332abed27ddaed1b44f1cb772bae94be27",
    "04fba018e3ed411b361f39de3cec584832fe9cd2",
    "4eacc562d6bc898ac74eab8f14bab897eae716b4",
    "c1d3ac97129f5edd8cc1d6e75f5f7afe4de6a34b",
    "fc57dda94830829b78a33eeea1010d3db7e9e0a9",
    "bfcbc5b2e84f8c3d291b2ee29e3d554aa18ebcec",
    "5bbd65070aa6b79ef190d8ec1c682eed14ee9095",
    "87b93ecb979d89c0d74be623a7808ec182a4b8cb",
    "370dbede4040d062159706439c03cb410063157e",
    "9724c996c369928e041da8bb996296c18b92beb9",
    "c909e2eed23c008907735b24a0b9eddbc3d107e7",
    "3af6d3ecea8d5959abe047f67fcfa0d985d54975",
    "19c9d03cd6bdf592cba69b5ce66a7b490a2f4d57",
    "70ae0bb9d5d597ed6aefa618a8ea4e27508473d5",
    "30e3a1271a2cade90001c8ac7aa6dc89fb9fd416",
    "53179068f82edd06defb3429b40a1c364e6b8f94",
    "7cb553b6f50ddc767cf794f58ed8151e5e588eed",
    "074d0c924724f3175c4fdd218e65435d79996b4a",
    "840954bb3338e59638a98e04a1cc72db04d67e48",
    "da1a1643661469de29e5c5d9a9e58c1cc44db62f",
    "a2678dd4489ed1d5d1bade89ac36f276a22d44c1",
    "52e26be1b953b168bbe903ef2f4ae1dc9892d60a",
    "7c8a0cb22745d59e35572c2669dae3470128eb3f",
    "a4fcdff7a33cfb446d26f9bf6a0e24998b0bbe42",
    "971a322988c351c8cfb16a42f05ab341bcadb4f5",
    "670138786f6b3d4fcdd723161d710005c0da954a",
    "3c18dafbef5845e1889f730528c9d520e27d2427",
    "db7da649969c0253c63b317a23d347dbf6bb7c4a",
    "73ee207ddec848c3f86cfceaba2fcb60e71ed9a2",
    "24c1133ef0e2706da6fcaacf8de7cf7c78ba34c7",
    "f9535eebe0ff944cd2c4bda7cb2ace821715df9a",
    "e54cd40942ea06636186131c4ed606c81f8be658",
    "60882be708883fd0e570b2764d441c4f9c24053d",
    "81b6e7ddaa47d3afa559b0cb582dcaa43b34db8a",
    "548588b1e9be94b0b2600ee8f8d0dea024823627",
    "5b1c378cfe0afadd70e70f9b368a7188230d22f1",
    "c2dbda9d1676e83b6983d4377f4fd7af50b1fbbb",
    "f207cec26a2436b4577ffd8f5ef3702cb7e08931",
    "34ecc514e321dfbf800d35014f15b6c02121e2d0",
    "b1725cbb38b89f3beac27cce427effd933e99395",
    "74da67e287bc4ef285b1b40944c5e931848c5143",
    "334a9b9edd04b3b8b2ee016a1a6bce59b141d580",
    "7e4131c1fa6088faf66eb479d32ed654a9c24779",
    "9fa6f7379cd8c4ebb08ee04568d2a1a671f5065b",
    "26a90250c15286e0fa14370aa0020c90974655ab",
    "85fa43e4cd025fda28605a42e5df8aeba3f2768b",
    "9c236cd255eacfab541fb72858e78d5b3ed138af",
    "f6665a3bcdcdbde856ba71d9f675efdb548a3b8e",
    "18eb7782ec43e37a3cc240504e6f5f194949922c",
    "d7603845e0f18df9916f104b7315de5d120e996f",
    "702fb63f27aad7f7c43199f2cb40effc5e7ff6d3",
    "73e80a2c6ff26f5c9c56687fb7489ca966b044b7",
    "65498c655ccaebefe0f8dcbc379899d4f61eac97",
    "ed84861794334d6f0d3ec0321d76490fb7eb1cfd",
    "1133abe6a525394b766dd9642a0e8dc95789c792",
    "e7bb8ccd39ac6d5b8c9c2d8300533e8cb28588a2",
    "c37918e96d8497060fefa9b62e0c53c217a5c5e7",
    "07a4b8db815e7dc67e4dae21b3e2c4edd1d95188",
    "7d3635e68d1443b56b6134cf40534d5cc7a2c69b",
    "1a5210b578f1863970a2f0ad2f67a1fd40dfbd7e",
    "2a5c73968720cc470b1ec1722929636c89676a1f",
    "c8e711a293f534bb3a146853f1af9f2c1a0cf882",
    "4256cb4846657456ca8bf1f7df4f0f9a119ad4b7",
    "88cff94ca29bb732fccd1b84c2a71e9952455376",
    "32d982a495efa44607541b01db5960da1f8358a8",
    "20cf242c4740f6a644f0162eba1b0457660848a2",
    "b7451a91d408aa8ab91691e2ce82844c98027fc5",
    "71ab0feb73c3ee84e9697ac9b816d0e98163c64f",
    "9ffb7d1c8695cc3bc1a4eb035145a64f0384f282",
    "54837f662ec3ff1a73019501a3bcd6f03bd753b5",
    "22e11b931e0453b3ef5b98ab9621d2263c25d335",
    "670c2593798dffafd07a081c0f3ee05611f0fb4d",
    "6c2d5e8cf5f3b2321d808b2f9221e93a59cbbcf4",
    "996b35c228d913c4cf0a107b10e07b70790f4bcd",
    "9006ba788f18f00bea3533fc50e27caf5ed684a2",
    "5704fd47ce7395ed502bfaadfa9b61077a9c4ce1",
    "f9dff882f474c5f17c54c14f45c3b73f7cec3787",
    "ac653b321e1260279eb9f58274b5dcf1e84f10d0",
    "332f82c5d9fb865b4731cb94f6506da656a6f95d",
    "019b482c9e3a01a57a4f2ea167e51c3da586cb0e",
    "64bbd197ea77c0660cd1e267c2d72b1788d6852f",
    "868c5a433afca682780212f39d5bbb454a84a67f",
    "bfa2b2abf16c86907481900e2c49fbaf17cf6053",
    "5ffda92f374a7f4a08431746cd9973046a4df151",
    "1ecad91f69922b0a0d3e68ea9d348c18331b23ef",
    "2d1b9828ce518a58926beddc62a3aab294e9eced",
    "cadfcf1817f1655b7ca6fcc55ec9291ec4025571",
    "b8757d4b552438d81afdd29ebec3bb13a4b906b6",
    "847dba431117d438cd823a22f5f08c366efc41ba",
    "37a5fc63e32df4f190930e25fc1c83aad855bfc7",
    "13df8275a7a5e0b4ca0b7b484b5f77a21794fe25",
    "3744254ff8d99c6d26a6f4e8c60ffad097807a4e",
    "4721ce280a71aa473b0d423d82f88f5f1e9d3d4f",
    "c4ed7e7526d0fffecd950efea2f5a1b95c645f4b",
    "ce65087a508c913303c2d3b4aed77fb368f3d0b4",
    "69dbf298b829b38fa53e17a5e4c4207938cb3afe",
    "29b9eba485786500c6d8c62a10785e1bdeb338ab",
    "99c297334769f3ad3dcfe70c11272926fdb77f35",
    "b1303865988704f647d99291868f4eeb856f35c6",
    "2b49c6dc2f2c102a63bb43b4e3a6d6916c14c41b",
    "688a1bb1e6c6fa2fa73977dfd54b3732c6f60c1e",
    "7577ef85f4b228a2c59592b324a6963449c6e4f5",
    "f29302258f2560333cabdc07547f1d713dd4e829",
    "de72bfacde1388faad206ad77b87de5bbf881e8a",
    "c05546dfc2f6d758d906cdd4fc0e65880abb8e22",
    "423cf753b1c82eb7445c60dd16a886e493891c33",
    "0820256d309e1c66fc8c57c0e117c4c70b83d33a",
    "6867018f2bb1d5d464d68252364d0b2dc0a48123",
    "4f789968012c757848a58e1511eb2494bc39b8c9",
    "226598caf11da1a4372346ca98fb9d513b32220d",
    "9582dcba8fe25b2dcdc4257b1b1e3de7abe6fc31",
    "c89a16c32f9ba9bb3417388f0376d5b686af0a59",
    "1e5218c493776e52017d6e0dc279441b47476013",
    "ab4e0daea6b9f89aeeaa7512249580f1572ca6c9",
    "5486d3830ea5fc89a3f27cb30efd5c36b0115cab",
    "f4a937329c643b814c23c88a57628a75156154cb",
    "b52c67567762480487f50d97abd14d92e75596bd",
    "5314edf160e668c80bae0eae3416a74722f7a987",
    "d225c373c395658583a3aa98f58491ffef1a65dd",
    "d119be08d6aacf4b06acd58805f2c24c14a68e88",
    "854dbf5d086d43a0bda400398190f3e54fd4c896",
    "c169ecab76b9cf02a44ed7c54bc42c2c0d1c997e",
    "a0f5bc64833ddf521fd4501105a1e682d3402856",
    "60c5e78ec331fb938e7ee5719da8880c8ef84535",
    "3e1881e0e366c9ea130d7c55a366ba00cba6f433",
    "66d1aa2aebbfb3eaf7ace4cfe45451009a8caac8",
    "9ae48b822c00bafe62bbf78e8f54c2a07eabef0d",
    "9402c8afd0173c4ef70f164a6e736eb2a4ec4e2d",
    "0ffe211ce08a4370f230a6aca461812dd24bca32",
    "49c9b830430fa8cf3e73e6a0ead34bd37fbb6635",
    "e11b73e661d12a9d51c86edfc0d570d131847609",
    "3333b3e6cf4c89be60b58e630c67bfb11debf496",
    "138a43012f6394fedd767d42273d7d821ff2f80e",
    "7fdd265157c81f5506de788082764b6a602b1e08",
    "dc652b39217c5c3e0c0db8e4775762f453e4c467",
    "06a67f6ba2dc6badcb90a26d5c4acb494cbb0d16",
    "aa9a32fa4b782ce92493cab6d9ef5c3b85bc4798",
    "3a77b46fffa39a85fa23ede03693b7f7893428a1",
    "4df87e883cf241dcff3e81ed3527e3f455da6d7f",
    "6efea2b0a37dac38f4d27807c1098821a70e1f47",
    "a7424a40a39000349dde6cc7b64e9acb5af57b08",
    "1f31fad3a62fa9ab35bddb97923cee5dbfeaee87",
    "abfe7cf43f27ceb25ebfd5767af39435ed7bc8a5",
    "532ced5fbcc7156d0c87bb986e5df49e67b29237",
    "0a82c443b24e1e4908fe7c413f4d46fc57d56e4e",
    "86e634996a6de1cc75d0b762a79d3c061146e54e",
    "4a1887d644dfbd9f849e094454cb65eb9791f3ad",
    "c9fa0ce4a253d3ea9fcaf3a7555096084b9d9df5",
    "734cd8d3f67cd4d0414feb4734eeba9b6afe88e4",
    "fc1d4378527c783aa58d078e760cb2e91a40f701",
    "80bb6f01ddfe8cfc8225dba7da79755c45b6e696",
    "16c78b5475ede8b9bc8d6848efe5d9769ef0628e",
    "8f27ffc6e8e8cc6650dbba56e697b118a6e3ff8d",
    "ea123dccf145aa275381fed3639946a2ea795670",
    "38ee38416691a5a90058bdf98dcd8d22263c8ca2",
    "80b7a656424cb6ec18de80da2dac254b83fda479",
    "a5f15b73b81555e4a363d7eadc85ec7bca12c9b8",
    "247b6e68af6be55aa166a3810580f1eef9a090e0",
    "a7ece5b3e8fe457aa0586d594088c30a1f7b4e11",
    "483f10a6d17c7bc5eaf11815b17b3917722cfccb",
    "2b0d413805d9a086c631deb8f49be1793f54ca71",
    "62a67d23f4d66406a6fc99a9dcf39d16319f84f1",
    "3102b537806471e3869cd4cab39846c755640ff9",
    "560a2257aaf27e8508847385b0dbc1c65ea2a002",
    "a3b1fedeaf00bb7c8dd1fa51221c20ddb683fee1",
    "68101a5e621fa496482297e5187f5a82129d8647",
    "4fd3c702c6c86a2415932643aa408399488d6307",
    "f114ad2cb90b8a7534b736c84460f71fda7ee5de",
    "be3e59c96f76442607bfff3ea46887801dd5fa2b",
    "4c41c69e5fd7449849a3b29793bc05eb0a07f085",
    "ad429fdc98b90d326e44e52680d576ec323f6591",
    "79e6c0d0d4ad13ca2717c31c9ab1f5adc90ce590",
    "4d839d7c2d67aae40ab464901edd155b424d6cba",
    "b85dec366570996e4d28875a17911d1fa9ed0b94",
    "1dc0de25687287502b6c7477c5f535fcf9097d7b",
    "82a099321d78a958ae60649304e2cf34e33379d1",
    "0b27f10768e6b685dc98fe0c8d1a947a00bca627",
    "29c1aa771e30c8ff835ce663f60d76ee751bdd78",
    "48acc4a0da8d622707de102f6fe93e0b2c0eb160",
    "9dc93068d2806dcec97f2301a8504ad4c9dac7b8",
    "cc6b8e641f76edcee1034ff46d970327918f2dfa",
}

function test_trivial_obj ()
    local obj = Filter:new("sha1")
    is("da39a3ee5e6b4b0d3255bfef95601890afd80709", bytes_to_hex(obj:result()))
    local obj = Filter:new("sha1")
    obj:add("")
    is("da39a3ee5e6b4b0d3255bfef95601890afd80709", bytes_to_hex(obj:result()))
end

function test_misc_sha1 ()
    for input, expected in pairs(misc_mapping) do
        local got = Filter.sha1(input)
        is(20, got:len())
        local input_msg = input:len() < 80 and input
                                            or input:sub(1, 80) .. "..."
        is(expected, bytes_to_hex(got),
           "SHA1 of " .. string.format("%q", input))
    end
end

function test_gradual_size_increase ()
    local input = ""
    local byte = 7

    for i = 1, #progressive_sha1_expected do
        local expected = progressive_sha1_expected[i]

        input = input .. string.char(byte)
        byte = (byte + 23) % 256
        assert(input:len() == i)

        local got = Filter.sha1(input)
        is(20, got:len())
        is(expected, bytes_to_hex(got),
           "SHA1 of " .. string.format("%q", input))
    end
end


-- test/50_base64.lua

local _ENV = TEST_CASE "test.base64"

local misc_mapping = {
    -- Test data from RFC 4648, section 10
    [""] = "",
    ["f"] = "Zg==",
    ["fo"] = "Zm8=",
    ["foo"] = "Zm9v",
    ["foob"] = "Zm9vYg==",
    ["fooba"] = "Zm9vYmE=",
    ["foobar"] = "Zm9vYmFy",
    -- Example from RFC 2617, section 2
    ["Aladdin:open sesame"] = "QWxhZGRpbjpvcGVuIHNlc2FtZQ==",
}

local single_byte_mapping = {
    ["\0"] = "AA==",
    ["\1"] = "AQ==",
    ["\2"] = "Ag==",
    ["\3"] = "Aw==",
    ["\4"] = "BA==",
    ["\5"] = "BQ==",
    ["\6"] = "Bg==",
    ["\7"] = "Bw==",
    ["\8"] = "CA==",
    ["\9"] = "CQ==",
    ["\10"] = "Cg==",
    ["\11"] = "Cw==",
    ["\12"] = "DA==",
    ["\13"] = "DQ==",
    ["\14"] = "Dg==",
    ["\15"] = "Dw==",
    ["\16"] = "EA==",
    ["\17"] = "EQ==",
    ["\18"] = "Eg==",
    ["\19"] = "Ew==",
    ["\20"] = "FA==",
    ["\21"] = "FQ==",
    ["\22"] = "Fg==",
    ["\23"] = "Fw==",
    ["\24"] = "GA==",
    ["\25"] = "GQ==",
    ["\26"] = "Gg==",
    ["\27"] = "Gw==",
    ["\28"] = "HA==",
    ["\29"] = "HQ==",
    ["\30"] = "Hg==",
    ["\31"] = "Hw==",
    ["\32"] = "IA==",
    ["\33"] = "IQ==",
    ["\34"] = "Ig==",
    ["\35"] = "Iw==",
    ["\36"] = "JA==",
    ["\37"] = "JQ==",
    ["\38"] = "Jg==",
    ["\39"] = "Jw==",
    ["\40"] = "KA==",
    ["\41"] = "KQ==",
    ["\42"] = "Kg==",
    ["\43"] = "Kw==",
    ["\44"] = "LA==",
    ["\45"] = "LQ==",
    ["\46"] = "Lg==",
    ["\47"] = "Lw==",
    ["\48"] = "MA==",
    ["\49"] = "MQ==",
    ["\50"] = "Mg==",
    ["\51"] = "Mw==",
    ["\52"] = "NA==",
    ["\53"] = "NQ==",
    ["\54"] = "Ng==",
    ["\55"] = "Nw==",
    ["\56"] = "OA==",
    ["\57"] = "OQ==",
    ["\58"] = "Og==",
    ["\59"] = "Ow==",
    ["\60"] = "PA==",
    ["\61"] = "PQ==",
    ["\62"] = "Pg==",
    ["\63"] = "Pw==",
    ["\64"] = "QA==",
    ["\65"] = "QQ==",
    ["\66"] = "Qg==",
    ["\67"] = "Qw==",
    ["\68"] = "RA==",
    ["\69"] = "RQ==",
    ["\70"] = "Rg==",
    ["\71"] = "Rw==",
    ["\72"] = "SA==",
    ["\73"] = "SQ==",
    ["\74"] = "Sg==",
    ["\75"] = "Sw==",
    ["\76"] = "TA==",
    ["\77"] = "TQ==",
    ["\78"] = "Tg==",
    ["\79"] = "Tw==",
    ["\80"] = "UA==",
    ["\81"] = "UQ==",
    ["\82"] = "Ug==",
    ["\83"] = "Uw==",
    ["\84"] = "VA==",
    ["\85"] = "VQ==",
    ["\86"] = "Vg==",
    ["\87"] = "Vw==",
    ["\88"] = "WA==",
    ["\89"] = "WQ==",
    ["\90"] = "Wg==",
    ["\91"] = "Ww==",
    ["\92"] = "XA==",
    ["\93"] = "XQ==",
    ["\94"] = "Xg==",
    ["\95"] = "Xw==",
    ["\96"] = "YA==",
    ["\97"] = "YQ==",
    ["\98"] = "Yg==",
    ["\99"] = "Yw==",
    ["\100"] = "ZA==",
    ["\101"] = "ZQ==",
    ["\102"] = "Zg==",
    ["\103"] = "Zw==",
    ["\104"] = "aA==",
    ["\105"] = "aQ==",
    ["\106"] = "ag==",
    ["\107"] = "aw==",
    ["\108"] = "bA==",
    ["\109"] = "bQ==",
    ["\110"] = "bg==",
    ["\111"] = "bw==",
    ["\112"] = "cA==",
    ["\113"] = "cQ==",
    ["\114"] = "cg==",
    ["\115"] = "cw==",
    ["\116"] = "dA==",
    ["\117"] = "dQ==",
    ["\118"] = "dg==",
    ["\119"] = "dw==",
    ["\120"] = "eA==",
    ["\121"] = "eQ==",
    ["\122"] = "eg==",
    ["\123"] = "ew==",
    ["\124"] = "fA==",
    ["\125"] = "fQ==",
    ["\126"] = "fg==",
    ["\127"] = "fw==",
    ["\128"] = "gA==",
    ["\129"] = "gQ==",
    ["\130"] = "gg==",
    ["\131"] = "gw==",
    ["\132"] = "hA==",
    ["\133"] = "hQ==",
    ["\134"] = "hg==",
    ["\135"] = "hw==",
    ["\136"] = "iA==",
    ["\137"] = "iQ==",
    ["\138"] = "ig==",
    ["\139"] = "iw==",
    ["\140"] = "jA==",
    ["\141"] = "jQ==",
    ["\142"] = "jg==",
    ["\143"] = "jw==",
    ["\144"] = "kA==",
    ["\145"] = "kQ==",
    ["\146"] = "kg==",
    ["\147"] = "kw==",
    ["\148"] = "lA==",
    ["\149"] = "lQ==",
    ["\150"] = "lg==",
    ["\151"] = "lw==",
    ["\152"] = "mA==",
    ["\153"] = "mQ==",
    ["\154"] = "mg==",
    ["\155"] = "mw==",
    ["\156"] = "nA==",
    ["\157"] = "nQ==",
    ["\158"] = "ng==",
    ["\159"] = "nw==",
    ["\160"] = "oA==",
    ["\161"] = "oQ==",
    ["\162"] = "og==",
    ["\163"] = "ow==",
    ["\164"] = "pA==",
    ["\165"] = "pQ==",
    ["\166"] = "pg==",
    ["\167"] = "pw==",
    ["\168"] = "qA==",
    ["\169"] = "qQ==",
    ["\170"] = "qg==",
    ["\171"] = "qw==",
    ["\172"] = "rA==",
    ["\173"] = "rQ==",
    ["\174"] = "rg==",
    ["\175"] = "rw==",
    ["\176"] = "sA==",
    ["\177"] = "sQ==",
    ["\178"] = "sg==",
    ["\179"] = "sw==",
    ["\180"] = "tA==",
    ["\181"] = "tQ==",
    ["\182"] = "tg==",
    ["\183"] = "tw==",
    ["\184"] = "uA==",
    ["\185"] = "uQ==",
    ["\186"] = "ug==",
    ["\187"] = "uw==",
    ["\188"] = "vA==",
    ["\189"] = "vQ==",
    ["\190"] = "vg==",
    ["\191"] = "vw==",
    ["\192"] = "wA==",
    ["\193"] = "wQ==",
    ["\194"] = "wg==",
    ["\195"] = "ww==",
    ["\196"] = "xA==",
    ["\197"] = "xQ==",
    ["\198"] = "xg==",
    ["\199"] = "xw==",
    ["\200"] = "yA==",
    ["\201"] = "yQ==",
    ["\202"] = "yg==",
    ["\203"] = "yw==",
    ["\204"] = "zA==",
    ["\205"] = "zQ==",
    ["\206"] = "zg==",
    ["\207"] = "zw==",
    ["\208"] = "0A==",
    ["\209"] = "0Q==",
    ["\210"] = "0g==",
    ["\211"] = "0w==",
    ["\212"] = "1A==",
    ["\213"] = "1Q==",
    ["\214"] = "1g==",
    ["\215"] = "1w==",
    ["\216"] = "2A==",
    ["\217"] = "2Q==",
    ["\218"] = "2g==",
    ["\219"] = "2w==",
    ["\220"] = "3A==",
    ["\221"] = "3Q==",
    ["\222"] = "3g==",
    ["\223"] = "3w==",
    ["\224"] = "4A==",
    ["\225"] = "4Q==",
    ["\226"] = "4g==",
    ["\227"] = "4w==",
    ["\228"] = "5A==",
    ["\229"] = "5Q==",
    ["\230"] = "5g==",
    ["\231"] = "5w==",
    ["\232"] = "6A==",
    ["\233"] = "6Q==",
    ["\234"] = "6g==",
    ["\235"] = "6w==",
    ["\236"] = "7A==",
    ["\237"] = "7Q==",
    ["\238"] = "7g==",
    ["\239"] = "7w==",
    ["\240"] = "8A==",
    ["\241"] = "8Q==",
    ["\242"] = "8g==",
    ["\243"] = "8w==",
    ["\244"] = "9A==",
    ["\245"] = "9Q==",
    ["\246"] = "9g==",
    ["\247"] = "9w==",
    ["\248"] = "+A==",
    ["\249"] = "+Q==",
    ["\250"] = "+g==",
    ["\251"] = "+w==",
    ["\252"] = "/A==",
    ["\253"] = "/Q==",
    ["\254"] = "/g==",
    ["\255"] = "/w==",
}

-- All of these contain illegal characters, but if those are ignored then
-- they all decode to 'frob'.
local bad_char_encodings = {
    "*ZnJvYg==",
    "Zn*JvYg==",
    "ZnJv*Yg==",
    "ZnJvYg*==",
    "ZnJvYg=*=",
    "ZnJvYg==*",
    "*Z*n*J*v*Y*g*=*=*",
    "\0\1\2\3\4\5\6\7\8\11\14\15ZnJvYg==",
    "\16\17\18\19\20\21\22\23\24\25\26\27\28\29\30\31ZnJvYg==",
    "!\"#$%&'()*,-.ZnJvYg==",
    ":;<>?@[\\]^_`{|}~\127ZnJvYg==",
    "\128\159\160\255ZnJvYg==",
}

function test_trivial_obj ()
    local obj = Filter:new("base64_encode")
    is("", obj:result())
    obj = Filter:new("base64_decode")
    is("", obj:result())
end

function test_encode ()
    for input, expected in pairs(misc_mapping) do
        is(expected, Filter.base64_encode(input),
           "encode value " .. string.format("%q", input))
    end
end

function test_decode ()
    for expected, input in pairs(misc_mapping) do
        is(expected, Filter.base64_decode(input),
           "decode value " .. string.format("%q", input))
    end
end

function test_single_byte_encode ()
    for input, expected in pairs(single_byte_mapping) do
        is(expected, Filter.base64_encode(input),
           "encode single byte " .. string.format("%q", input))
    end
end

function test_single_byte_decode ()
    for expected, input in pairs(single_byte_mapping) do
        is(expected, Filter.base64_decode(input),
           "decode to single byte from " .. string.format("%q", input))
    end
end

function test_decode_with_whitespace ()
    is("", Filter.base64_decode("  \t\n\r \13\10 "), "just whitespace")
    is("xyz", Filter.base64_decode("eHl6   "), "whitespace after")
    is("xyz", Filter.base64_decode(" e  H l 6 "),
       "whitespace intermingled")
    is("xy", Filter.base64_decode(" e  H k = "),
       "whitespace intermingled, one padding char")
    is("x", Filter.base64_decode(" e  A = = "),
       "whitespace intermingled, two padding chars")
end

function test_big_whitespace ()
    is("", Filter.base64_decode((" "):rep(8192)), "bufsize whitespace")

    for i = 8187, 8193 do
        is("xyz", Filter.base64_decode((" "):rep(i) .. "eHl6"),
           i .. " bytes of whitespace plus block")
    end

    for i = 8187, 8193 do
        is("xyz", Filter.base64_decode("eHl6" .. (" "):rep(i)),
           "block plus " .. i .. " bytes of whitespace")
    end
end

function test_whitespace_not_allowed ()
    local encoded_cases = { "eHl6", "eHk=", "eA==", "" }
    local options = { allow_whitespace = false }
    for _, encoded in ipairs(encoded_cases) do
        for space_pos = 0, encoded:len() do
            local input = encoded:sub(1, space_pos) .. " " ..
                          encoded:sub(space_pos + 1)
            assert_error("whitespace not allowed in [" .. input .. "]",
                         function () Filter.base64_decode(input, options) end)
        end
    end
end

function test_oo_encode ()
    local obj = Filter:new("base64_encode")
    obj:add("Aladdin")
    obj:add(":")
    obj:add("open sesame")
    is("QWxhZGRpbjpvcGVuIHNlc2FtZQ==", obj:result())
end

function test_no_padding ()
    for input, expected in pairs(misc_mapping) do
        expected = expected:gsub("=+$", "", 1)
        is(expected, Filter.base64_encode(input, { include_padding = false }),
           "encode value without padding " .. string.format("%q", input))
    end
end

local function test_with_line_breaking (line_ending)
    for max_line_len = 1, 20 do
        for input, expected in pairs(misc_mapping) do
            local dots = ("."):rep(max_line_len)
            expected = expected:gsub("(" .. dots .. ")", "%1" .. line_ending)
            if expected ~= "" and not expected:find(line_ending .. "$") then
                expected = expected .. line_ending
            end
            local got = Filter.base64_encode(input, {
                line_ending = line_ending,
                max_line_length = max_line_len,
            })
            local desc = "encode value with " .. max_line_len ..
                         " bytes per line, line ending " ..
                         string.format("%q", line_ending) .. ", input " ..
                         string.format("%q", input)
            is(expected, got, desc)
        end
    end
end

function test_eol ()
    test_with_line_breaking("\13\10")
    test_with_line_breaking("\13")
    test_with_line_breaking("\10")
    test_with_line_breaking("foobar baz")
    test_with_line_breaking("")
end

function test_eol_defaults ()
    local input = ("foobar"):rep(10)
    local result = Filter.base64_encode(input)
    is(("Zm9vYmFy"):rep(10), result, "no line breaking")
    local result = Filter.base64_encode(input, { line_ending = "\10" })
    is(("Zm9vYmFy"):rep(9) .. "Zm9v\10YmFy\10", result, "default line len")
    local result = Filter.base64_encode(input, { max_line_length = 76 })
    is(("Zm9vYmFy"):rep(9) .. "Zm9v\13\10YmFy\13\10", result,
       "default line ending, explicit default line len")
    local result = Filter.base64_encode(input, { max_line_length = 40 })
    is(("Zm9vYmFy"):rep(5) .. "\13\10" .. ("Zm9vYmFy"):rep(5) .. "\13\10",
       result, "default line ending, non-default line len")

    is("ZnJvYg==", Filter.base64_encode("frob", { include_padding = true }),
       "explicit default include_padding")
end

function test_missing_padding_error ()
    assert_error("spare char", function () Filter.base64_decode("e") end)

    assert_error("missing '==' in first block",
                 function () Filter.base64_decode("eA") end)
    assert_error("input has '=' instead of '==' in first block",
                 function () Filter.base64_decode("eA=") end)
    assert_error("missing '=' in first block",
                 function () Filter.base64_decode("eHk") end)

    assert_error("missing '==' after first block",
                 function () Filter.base64_decode("eHl6eg") end)
    assert_error("input has '=' instead of '==' in second block",
                 function () Filter.base64_decode("eHl6eg=") end)
    assert_error("missing '=' after first block",
                 function () Filter.base64_decode("eHl6enk") end)
end

function test_missing_padding_ok ()
    local options = { allow_missing_padding = true }

    -- This is always wrong, even if you're lenient about missing padding.
    assert_error("spare char",
                 function () Filter.base64_decode("e", options) end)

    is("x", Filter.base64_decode("eA", options),
       "missing '==' in first block")
    is("x", Filter.base64_decode("eA=", options),
       "input has '=' instead of '==' in first block")
    is("xy", Filter.base64_decode("eHk", options),
       "missing '=' in first block")

    is("xyzz", Filter.base64_decode("eHl6eg", options),
       "missing '==' after first block")
    is("xyzz", Filter.base64_decode("eHl6eg=", options),
       "input has '=' instead of '==' in second block")
    is("xyzzy", Filter.base64_decode("eHl6enk", options),
       "missing '=' after first block")
end

function test_padding_in_wrong_place ()
    assert_error("padding char first in block, end of input",
                 function () Filter.base64_decode("=") end)
    assert_error("padding char second in block, end of input",
                 function () Filter.base64_decode("e=") end)

    assert_error("padding char first in block, input remaining",
                 function () Filter.base64_decode("=eHl6") end)
    assert_error("padding char second in block, input remaining",
                 function () Filter.base64_decode("e=eHl6") end)

    assert_error("one padding char ok, but more input afterwards",
                 function () Filter.base64_decode("eHk=eHl6") end)
    assert_error("two padding chars ok, but more input afterwards",
                 function () Filter.base64_decode("eA==eHl6") end)

    assert_error("padding char followed by alphabet char instead of padding",
                 function () Filter.base64_decode("eA=X") end)
end

local function test_spare_bits (input)
    assert_error("spare bits set in " .. input,
                 function () Filter.base64_decode(input) end)
end

function test_spare_bits_set ()
    test_spare_bits("eHl=")
    test_spare_bits("eHm=")
    test_spare_bits("eHn=")

    local last = {"B","C","D","E","F","G","H","I","J","K","L","M","N","O","P"}
    for _, letter in ipairs(last) do
        test_spare_bits("e" .. letter .. "==")
    end
end

function test_bad_chars_error ()
    for _, input in ipairs(bad_char_encodings) do
        assert_error("bad characters detected: " .. string.format("%q", input),
                     function () Filter.base64_decode(input) end)
    end
end

function test_bad_chars_error ()
    local options = { allow_invalid_characters = true }
    for _, input in ipairs(bad_char_encodings) do
        is("frob", Filter.base64_decode(input, options),
           "bad characters skipped: " .. string.format("%q", input))
    end
end

function test_bad_usage ()
    local options = { line_ending = true }
    assert_error("bad type for line_ending option",
                 function () Filter.base64_encode("foo", options) end)

    options = { max_line_length = "bad" }
    assert_error("bad type for max_line_length option",
                 function () Filter.base64_encode("foo", options) end)
    options = { max_line_length = 0 }
    assert_error("max_line_length must not be zero",
                 function () Filter.base64_encode("foo", options) end)
    options = { max_line_length = -23 }
    assert_error("max_line_length must not be negative",
                 function () Filter.base64_encode("foo", options) end)
end


-- test/50_hex.lua

local _ENV = TEST_CASE "test.hex"

local misc_binary_data = {
    "",
    "x\0y",
    "\0\15\240\255",
}

function test_trivial_obj ()
    local obj = Filter:new("hex_lower")
    is("", obj:result())
    obj = Filter:new("hex_upper")
    is("", obj:result())
end

function test_misc_lowercase ()
    for num, input in ipairs(misc_binary_data) do
        local expected = bytes_to_hex(input)
        is(expected, Filter.hex_lower(input),
           "lowercase hex for binary test datum " .. num)
    end
end

function test_misc_uppercase ()
    for num, input in ipairs(misc_binary_data) do
        local expected = bytes_to_hex(input):upper()
        is(expected, Filter.hex_upper(input),
           "uppercase hex for binary test datum " .. num)
    end
end

function test_single_byte_lowercase ()
    for code = 0, 255 do
        local input = string.char(code)
        local expected = bytes_to_hex(input)
        is(expected, Filter.hex_lower(input),
           "lowercase hex for single byte " .. code)
    end
end

function test_single_byte_uppercase ()
    for code = 0, 255 do
        local input = string.char(code)
        local expected = bytes_to_hex(input):upper()
        is(expected, Filter.hex_upper(input),
           "uppercase hex for single byte " .. code)
    end
end

function test_hex_big ()
    local input = ""
    for code = 0, 255 do input = input .. string.char(code) end
    local reps = 35     -- big enough to blow the input and output buffers
    input = input:rep(reps)

    local expected = bytes_to_hex(input)
    local got = Filter.hex_lower(input)
    is(reps * 256 * 2, got:len(), "lowercase hex of large amount of input")
    is(expected, got, "lowercase hex of large amount of input")

    expected = expected:upper()
    got = Filter.hex_upper(input)
    is(reps * 256 * 2, got:len(), "uppercase hex of large amount of input")
    is(expected, got, "uppercase hex of large amount of input")
end

function test_decode_hex ()
    -- Create some test input and expected binary data that matches.
    local input = " \t\n\r "
    local expected = ""
    for code = 0, 255 do
        input = input .. string.format("%02x%02X", code, code)
        expected = expected .. string.char(code, code)
    end
    input = input .. "\n"
    for _, data in ipairs(misc_binary_data) do
        input = input .. Filter.hex_lower(data) ..
                         Filter.hex_upper(data) .. "\n"
        expected = expected .. data .. data
    end

    -- Enough repetitions to make it big enough to deal with in chunks.
    input = input:rep(100)
    expected = expected:rep(100)

    is(expected, Filter.hex_decode(input))

    -- Test for digits of the same byte separated by whitespace.
    is("xyz", Filter.hex_decode("7 87 9\n7\na\n"))

    -- Check empty input.
    is("", Filter.hex_decode(""))
    is("", Filter.hex_decode(" \n\t \r "))
end

function test_decode_hex_bad ()
    assert_error("bad character instead of first digit",
                 function () Filter.hex_decode("01 *3 78") end)
    assert_error("bad character instead of second digit",
                 function () Filter.hex_decode("01 2* 78") end)
    assert_error("odd number of digits",
                 function () Filter.hex_decode("01 23 7 ") end)
end


-- test/50_pctenc.lua

local _ENV = TEST_CASE "test.pctenc"

local default_charset_mapping = {
    [""] = "",
    -- All unreserved characters
    ["ABCDEFGHIJKLMNOPQRSTUVWXYZ"] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ",
    ["abcdefghijklmnopqrstuvwxyz"] = "abcdefghijklmnopqrstuvwxyz",
    ["0123456789"] = "0123456789",
    ["-._~"] = "-._~",
    -- All reserved characters
    [":/?#[]@"] = "%3A%2F%3F%23%5B%5D%40",
    ["!$&'()*+,;="] = "%21%24%26%27%28%29%2A%2B%2C%3B%3D",
    -- Percent signs
    ["%%%"] = "%25%25%25",
    ["%25_%2525_%%2525"] = "%2525_%252525_%25%252525",
    -- All control characters
    ["\0\1\2\3\4\5\6\7\8\9\10\11\12\13\14\15"]
        = "%00%01%02%03%04%05%06%07%08%09%0A%0B%0C%0D%0E%0F",
    ["\16\17\18\19\20\21\22\23\24\25\26\27\28\29\30\31"]
        = "%10%11%12%13%14%15%16%17%18%19%1A%1B%1C%1D%1E%1F",
    -- Other characters, not legal in a URI
    [" a % X  X "] = "%20a%20%25%20X%20%20X%20",
    ["\"<>\\^`{|}\127"] = "%22%3C%3E%5C%5E%60%7B%7C%7D%7F",
    ["\128\129\130\131\132\133\134\135\136\137\138\139\140\141\142\143"]
        = "%80%81%82%83%84%85%86%87%88%89%8A%8B%8C%8D%8E%8F",
    ["\144\145\146\147\148\149\150\151\152\153\154\155\156\157\158\159"]
        = "%90%91%92%93%94%95%96%97%98%99%9A%9B%9C%9D%9E%9F",
    ["\160\161\162\163\164\165\166\167\168\169\170\171\172\173\174\175"]
        = "%A0%A1%A2%A3%A4%A5%A6%A7%A8%A9%AA%AB%AC%AD%AE%AF",
    ["\176\177\178\179\180\181\182\183\184\185\186\187\188\189\190\191"]
        = "%B0%B1%B2%B3%B4%B5%B6%B7%B8%B9%BA%BB%BC%BD%BE%BF",
    ["\192\193\194\195\196\197\198\199\200\201\202\203\204\205\206\207"]
        = "%C0%C1%C2%C3%C4%C5%C6%C7%C8%C9%CA%CB%CC%CD%CE%CF",
    ["\208\209\210\211\212\213\214\215\216\217\218\219\220\221\222\223"]
        = "%D0%D1%D2%D3%D4%D5%D6%D7%D8%D9%DA%DB%DC%DD%DE%DF",
    ["\224\225\226\227\228\229\230\231\232\233\234\235\236\237\238\239"]
        = "%E0%E1%E2%E3%E4%E5%E6%E7%E8%E9%EA%EB%EC%ED%EE%EF",
    ["\240\241\242\243\244\245\246\247\248\249\250\251\252\253\254\255"]
        = "%F0%F1%F2%F3%F4%F5%F6%F7%F8%F9%FA%FB%FC%FD%FE%FF",
    -- Example from RFC 2986, section 2.5
    ["\195\128_\227\130\162"] = "%C3%80_%E3%82%A2",
}

function test_empty ()
    is("", Filter.percent_encode(""))
    is("", Filter.percent_decode(""))
end

function test_encode ()
    for input, expected in pairs(default_charset_mapping) do
        is(expected, Filter.percent_encode(input),
           "encode value " .. string.format("%q", input))
    end
end

function test_decode ()
    for expected, input in pairs(default_charset_mapping) do
        is(expected, Filter.percent_decode(input),
           "decode value " .. string.format("%q", input))
    end
end

function test_encode_nondefault_safe_bytes ()
    local options = { safe_bytes = "acf" }
    is("%20a%62c%64%65f%67%25", Filter.percent_encode(" abcdefg%", options))
    options = { safe_bytes = 23 }
    is("%3123%34", Filter.percent_encode("1234", options))
end

function test_decode_hex_case_insensitive ()
    is("\171/\255\255\255", Filter.percent_decode("%ab%2f%fF%Ff%ff"))
end

function test_big ()
    for bufsize = 8185, 8194 do
        local padding = ("x"):rep(bufsize)
        is(padding .. "^\"%", Filter.percent_decode(padding .. "%5e%22%25"))
        is(padding .. "%5E%22%25", Filter.percent_encode(padding .. "^\"%"))
    end
end

function test_bad_hex_detected ()
    local options = {}
    assert_error("no hex digits",
                 function () Filter.percent_decode("foo%", options) end)
    assert_error("only one hex digit",
                 function () Filter.percent_decode("foo%3", options) end)
    assert_error("bad first hex digit",
                 function () Filter.percent_decode("foo%G3", options) end)
    assert_error("bad second hex digit",
                 function () Filter.percent_decode("foo%3g", options) end)
end

function test_bad_usage ()
    local options = { safe_bytes = true }
    assert_error("bad value for 'safe_bytes' option",
                 function () Filter.percent_encode("x%y", options) end)
    options = { safe_bytes = "o%" }
    assert_error("can't declare '%' to be safe char",
                 function () Filter.percent_encode("x%y", options) end)
    options = { safe_bytes = "yxy" }
    assert_error("don't repeat declaration of same safe byte",
                 function () Filter.percent_encode("x%y", options) end)
end


-- test/50_qp.lua

local _ENV = TEST_CASE "test.qp"

local x70 = ("x"):rep(70)

-- This test data was borrowed from the Perl module MIME::Base64 version 3.07.
local mapping_both_ways = {
    -- plain ascii should not be encoded
    [""] = "",
    ["quoted printable"]  =
        "quoted printable=\13\10",

    -- 8-bit chars should be encoded
    ["v\229re kj\230re norske tegn b\248r \230res"] =
        "v=E5re kj=E6re norske tegn b=F8r =E6res=\13\10",

    -- trailing space should be encoded
    ["  "] = " =20=\13\10",
    ["\tt\t"] = "\tt=09=\13\10",
    ["test  \ntest\n\t \t \n"] = "test =20\13\10test\13\10\t \t=20\13\10",

    -- "=" is special an should be decoded
    ["=30\n"] = "=3D30\13\10",
    ["\0\2550"] = "=00=FF0=\13\10",

    -- Very long lines should be broken (not more than 76 chars
    ["The Quoted-Printable encoding is intended to represent data that largly consists of octets that correspond to printable characters in the ASCII character set."] =
        "The Quoted-Printable encoding is intended to represent data that largly con=\13\10" ..
        "sists of octets that correspond to printable characters in the ASCII charac=\13\10" ..
        "ter set.=\13\10",

    -- Long lines after short lines were broken through 2.01.
    ["short line\10" ..
     "In America, any boy may become president and I suppose that's just one of the risks he takes. -- Adlai Stevenson"] =
        "short line\13\10" ..
        "In America, any boy may become president and I suppose that's just one of t=\13\10" ..
        "he risks he takes. -- Adlai Stevenson=\13\10",

    -- My (roderick@argon.org) first crack at fixing that bug failed for
    -- multiple long lines.
    ["College football is a game which would be much more interesting if the faculty played instead of the students, and even more interesting if the\n" ..
        "trustees played.  There would be a great increase in broken arms, legs, and necks, and simultaneously an appreciable diminution in the loss to humanity. -- H. L. Mencken"] =
        "College football is a game which would be much more interesting if the facu=\13\10" ..
        "lty played instead of the students, and even more interesting if the\13\10" ..
        "trustees played.  There would be a great increase in broken arms, legs, and=\13\10" ..
        " necks, and simultaneously an appreciable diminution in the loss to humanit=\13\10" ..
        "y. -- H. L. Mencken=\13\10",

    -- Don't break a line that's near but not over 76 chars.
    [x70.."!23"]       = x70.."!23=\13\10",
    [x70.."!234"]      = x70.."!234=\13\10",
    [x70.."!2345"]     = x70.."!2345=\13\10",
    [x70.."!23456"]    = x70.."!2345=\13\10".."6=\13\10",
    [x70.."!234567"]   = x70.."!2345=\13\10".."67=\13\10",
    [x70.."!23456="]   = x70.."!2345=\13\10".."6=3D=\13\10",
    [x70.."!23\n"]     = x70.."!23\13\10",
    [x70.."!234\n"]    = x70.."!234\13\10",
    [x70.."!2345\n"]   = x70.."!2345\13\10",
    [x70.."!23456\n"]  = x70.."!23456\13\10",
    [x70.."!234567\n"] = x70.."!2345=\13\10".."67\13\10",
    [x70.."!23456=\n"] = x70.."!2345=\13\10".."6=3D\13\10",

    -- Not allowed to break =XX escapes using soft line break
    [x70.."===xxxxx"]  = x70.."=3D=\13\10=3D=3Dxxxxx=\13\10",
    [x70.."!===xxxx"]  = x70.."!=3D=\13\10=3D=3Dxxxx=\13\10",
    [x70.."!2===xxx"]  = x70.."!2=3D=\13\10=3D=3Dxxx=\13\10",
    [x70.."!23===xx"]  = x70.."!23=\13\10=3D=3D=3Dxx=\13\10",
    [x70.."!234===x"]  = x70.."!234=\13\10=3D=3D=3Dx=\13\10",
    [x70.."!2=\n"]     = x70.."!2=3D\13\10",
    [x70.."!23=\n"]    = x70.."!23=3D\13\10",
    [x70.."!234=\n"]   = x70.."!234=\13\10=3D\13\10",
    [x70.."!2345=\n"]  = x70.."!2345=\13\10=3D\13\10",
    [x70.."!23456=\n"] = x70.."!2345=\13\10".."6=3D\13\10",
    --                               ^
    --                       70123456|
    --                              max
    --                           line width

    -- some extra special cases we have had problems with
    [x70.."!2=x=x"] = x70.."!2=3D=\13\10x=3Dx=\13\10",
    [x70.."!2345"..x70.."!2345"..x70.."!23456\n"]
        = x70.."!2345=\13\10"..x70.."!2345=\13\10"..x70.."!23456\13\10",

    -- trailing whitespace
    ["foo \t "] = "foo \t=20=\13\10",
    ["foo\t \10 \t"] = "foo\t=20\13\10 =09=\13\10",
}

local mapping_encoded_to_decoded = {
    -- Some extra testing for a case that was wrong until libwww-perl-5.09
    ["foo  \n\nfoo =\n\nfoo=20\n\n"] = "foo\n\nfoo \nfoo \n\n",

    -- Same test but with "\r\n" terminated lines
    ["foo  \r\n\r\nfoo =\r\n\r\nfoo=20\r\n\r\n"] = "foo\n\nfoo \nfoo \n\n",

    -- Trailing whitespace
    ["foo  "] = "foo  ",
    ["foo  \n"] = "foo\n",
    ["foo = \t\32\nbar\t\32\n"] = "foo bar\n",
    ["foo = \t\32\r\nbar\t\32\r\n"] = "foo bar\n",
    ["foo = \t\32\n"] = "foo ",
    ["foo = \t\32\r\n"] = "foo ",
    ["foo = \t\32y\r\n"] = "foo = \t\32y\n",
    ["foo =xy\n"] = "foo =xy\n",
}

function test_encode ()
    for input, expected in pairs(mapping_both_ways) do
        is(expected, Filter.qp_encode(input),
           "encode value " .. string.format("%q", input))
    end
end

function test_decode ()
    for expected, input in pairs(mapping_both_ways) do
        is(expected, Filter.qp_decode(input),
           "decode value " .. string.format("%q", input))
    end
end

function test_alternate_line_break ()
    is(x70.."!2345=***"..x70.."***",
       Filter.qp_encode(x70.."!2345"..x70.."\n", { line_ending = "***" }))
end

function test_no_line_breaks ()
--    is(x70.."!2345"..x70.."=0A",
--       Filter.qp_encode(x70.."!2345"..x70.."\n", { line_ending = "" }))
end

-- Test binary encoding
--"foo", undef, 1) eq "foo=\n";
--"foo\nbar\r\n", undef, 1) eq "foo=0Abar=0D=0A=\n";
--join("", map chr, 0..255), undef, 1) eq <<'EOT';
--=00=01=02=03=04=05=06=07=08=09=0A=0B=0C=0D=0E=0F=10=11=12=13=14=15=16=17=18=
--=19=1A=1B=1C=1D=1E=1F !"#$%&'()*+,-./0123456789:;<=3D>?@ABCDEFGHIJKLMNOPQRS=
--TUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~=7F=80=81=82=83=84=85=86=87=88=
--=89=8A=8B=8C=8D=8E=8F=90=91=92=93=94=95=96=97=98=99=9A=9B=9C=9D=9E=9F=A0=A1=
--=A2=A3=A4=A5=A6=A7=A8=A9=AA=AB=AC=AD=AE=AF=B0=B1=B2=B3=B4=B5=B6=B7=B8=B9=BA=
--=BB=BC=BD=BE=BF=C0=C1=C2=C3=C4=C5=C6=C7=C8=C9=CA=CB=CC=CD=CE=CF=D0=D1=D2=D3=
--=D4=D5=D6=D7=D8=D9=DA=DB=DC=DD=DE=DF=E0=E1=E2=E3=E4=E5=E6=E7=E8=E9=EA=EB=EC=
--=ED=EE=EF=F0=F1=F2=F3=F4=F5=F6=F7=F8=F9=FA=FB=FC=FD=FE=FF=
--EOT

function test_bad_usage ()
    local options = { line_ending = true }
    assert_error("bad type for line_ending option",
                 function () Filter.qp_encode("foo", options) end)
end


-- test/99_finish.lua

lunit.main(arg)
if lunit.stats.failed > 0 or lunit.stats.errors > 0 then
    os.exit(false, true)
end
